{% extends 'players/base.html' %}

{% block content %}
    <h1>Welcome to AI:MMO!</h1>
    <p>AI:MMO is a <strong>M</strong>assively <strong>M</strong>ultiplayer <strong>O</strong>nline game, where players
        create <strong>A</strong>rtificially <strong>I</strong>ntelligent programs to play on their behalf.</p>
    <p>Are you interested in learning more about AI:MMO and would like to try it out? <a href="https://www.codeforlife.education/play/aimmo/">Find out more and register your interest on our main website.</a></p>
{% endblock %}
